<?php
/**
 * export_wp_data.php
 * WordPressの記事や写真のデータをまるごとエクスポートするPHPスクリプトです。
 *
 * ■使い方
 * 本スクリプトをwordpressディレクトリ直下に置くか、ドキュメントルート外に置いて引数でwp-load.phpのパスを指定して実行します。
 *
 * ■引数
 * -w(オプション) : wp-load.phpのパス。ない場合は同ディレクトリにwp-load.phpがあるかをさがしにいきます。
 * -e(オプション) : エクスポート先ディレクトリ。ない場合は同ディレクトリに出力します。
 *
 * ■セキュリティについて
 * 本スクリプトは2つの最低限の以下のセキュリティがあります。
 * 1. このスクリプトがCLIからしか実行出来ない事
 * 2. エクスポートするzipファイル名に秒単位の日時を付与してファイル名を類推しにくい事
 * これによりすぐには悪さは出来ないようになっていますが完璧ではありません。
 * ドキュメントルート外に置いて引数指定で実行して、役目を終えたらいずれにファイルもすぐに消すとより安全です。
 */
if (php_sapi_name() != 'cli') {
  die("only cli." . PHP_EOL);
}
echo_msg("start export_wp_data");

/**
 * パラメータの取得と各種エラーチェック
 */
$options      = getopt("w::e::");
$wp_load_path = (isset($options["w"])) ? $options["w"] : false; // 指定ない場合はこのディレクトリの想定
$export_dir   = (isset($options["e"])) ? $options["e"] : false; // 指定ない場合はこのディレクトリ

// wp-load.phpファイルチェック
if ($wp_load_path) { // 指定があった時はパスチェック
  $wp_load_path = realpath($wp_load_path);  // 相対パスの場合は実行時の位置をもとに絶対パスにする
  if (!$wp_load_path) {
    die("パラメータ指定のwp-load.phpが存在しません" . PHP_EOL);
  }
} else {
  $wp_load_path = realpath(__DIR__ . "/wp-load.php"); // 指定ない場合このスクリプトのディレクトリを探す
  if (!$wp_load_path) {
    die("wp-load.phpが存在するディレクトリではありません" . PHP_EOL);
  }
}

// wp-load.phpを読み込めばDBの接続情報も読み込める
define('WP_DEBUG', false);  // エクスポート時にちょっとしたWPデバッグ情報は不要なのでfalseにする
require_once $wp_load_path;

// WPはUTC標準なのでここでは日本時間とする
date_default_timezone_set('Asia/Tokyo');

/* 定数を文字列内で扱いやすいように変数化する */
$host = DB_HOST;
$user = DB_USER;
$pass = DB_PASSWORD;
$db   = DB_NAME;

// DB接続確認
try {
  new PDO('mysql:host=' . $host . ';dbname=' . $db, $user, $pass);
} catch (PDOException $e) {
  die("MySQLに接続出来ません" . PHP_EOL);
}

// 引数でエクスポートディレクトリの指定がある場合はそこに出力する
if ($export_dir) { // 指定があった時はパスチェック
  $export_dir = realpath($export_dir);  // 絶対パスにする
  if (!$export_dir) {
    die("指定のエクスポートディレクトリが存在しません" . PHP_EOL);
  }
} else {
  // 引数でエクスポートディレクトリの指定がない場合は本スクリプトと同ディレクトリにする
  $export_dir = __DIR__;
}

// uploads存在チェック
if (!file_exists(WP_CONTENT_DIR . "/uploads")) {
  die("wp-contentディレクトリ直下にuploadsディレクトリが存在しません" . PHP_EOL);
}

// このサイトドメイン持ってきてzipファイル名にする
$wp_host = parse_url(get_site_url(), PHP_URL_HOST);

/* uploadsディレクトリをzip化する */
// zipコマンドは絶対パスでやるとそこまでのパスが入っちゃうのでそのディレクトリに移動する
// wp-loads.phpを読み込むとWP_CONTENT_DIRが定義されているのでそれを使う
echo_msg("> cd " . WP_CONTENT_DIR);
chdir(WP_CONTENT_DIR);

// -r サブディレクトリも再帰的にzip化する
// -q 動作中のメッセージを表示しない
$zippath = "{$export_dir}/wp_data-{$wp_host}-" . date("YmdHis") . ".zip";  // データディレクトリに生成するzipパス
exec_cmd("zip -rq {$zippath} ./uploads"); // wp-content直下のuploadsディレクトリをzipに突っ込む

if (!file_exists($zippath)) {
  die("uploadsディレクトリをzip出来ません" . PHP_EOL);
}

/* mysqldumpで指定DBのデータを出力する */
echo_msg("> cd {$export_dir}");
chdir($export_dir);  // uploads手前から移動
$sql_file = "wordpress_db.sql";
// --single-transaction サーバーからデータをダンプする前に BEGIN SQL ステートメントを発行（DBロックしない）
// --no-tablespaces CREATE LOGFILE GROUP ステートメントおよび CREATE TABLESPACE ステートメントを出力に書き出さない（これがないとWARNING出る）
exec_cmd("mysqldump -h {$host} -u {$user} -p{$pass} --single-transaction --no-tablespaces {$db} > {$sql_file}");

if (!file_exists($sql_file)) {
  die("SQLファイルをダンプ出来ません" . PHP_EOL);
}

/* sqlファイルをzipに追加する */
exec_cmd("zip -q {$zippath} {$sql_file}");

/* sqlファイルはzipしたら不要なので消去 */
exec_cmd("rm {$sql_file}");

echo_msg("success : export {$zippath}");

// 最後に改行つけるだけのecho
function echo_msg($msg) {
  echo $msg . PHP_EOL;
}

// コマンド実行関数
function exec_cmd($cmd) {
  echo_msg("> {$cmd}");
  system($cmd, $return);  // exec()ではなくsystem()を使っているのは実行結果を出力してくれるから
  return $return;
}