<?php
/**
 * export_wp_data.php
 * WordPressの記事や写真のデータをまるごとエクスポートするPHPスクリプトです。
 *
 * ■使い方
 * 本スクリプトをwordpressディレクトリ直下に置くか、ドキュメントルート外に置いて引数でwp-load.phpのパスを指定して実行します。
 *
 * ■引数
 * -w(必須) : wp-contentディレクトリのパス。
 * -h(必須) : MySQLのhostname
 * -u(必須) : MySQLのusername
 * -p(必須) : MySQLのpassword
 * -d(必須) : MySQLのdatabase
 * -e(オプション)：エクスポート先ディレクトリ
 *
 * ■セキュリティについて
 * 本スクリプトは2つの最低限の以下のセキュリティがあります。
 * 1. このスクリプトがCLIからしか実行出来ない事
 * 2. エクスポートするzipファイル名に秒単位の日時を付与してファイル名を類推しにくい事
 * これによりすぐには悪さは出来ないようになっていますが完璧ではありません。
 * ドキュメントルート外に置いて引数指定で実行して、役目を終えたらいずれにファイルもすぐに消すとより安全です。
 */
if (php_sapi_name() != 'cli') {
  die("only cli." . PHP_EOL);
}

echo_msg("start export_wp_data");

// まずはこのスクリプトの場所に移動
echo_msg("> cd " . __DIR__);
chdir(__DIR__);

/**
 * パラメータの取得
 */
$options = getopt("w:h:u:p:d:e::");
// 必須パラメータ
$wp_content_dir = (isset($options["w"])) ? $options["w"] : die("wp-contentディレクトリパスのパラメータは必須です" . PHP_EOL);
$host           = (isset($options["h"])) ? $options["h"] : die("DB接続情報のhostは必須です。" . PHP_EOL);
$user           = (isset($options["u"])) ? $options["u"] : die("DB接続情報のuserは必須です" . PHP_EOL);
$pass           = (isset($options["p"])) ? $options["p"] : die("DB接続情報のpassは必須です" . PHP_EOL);
$db             = (isset($options["d"])) ? $options["d"] : die("DB接続情報のdbは必須です" . PHP_EOL);
// オプションパラメータ
$export_dir = (isset($options["e"])) ? $options["e"] : false; // 指定ない場合はこのディレクトリ

// wp-contentチェック
$wp_content_dir = realpath($wp_content_dir);
if (!$wp_content_dir) {
  die("wp-contentディレクトリの指定が正しくありません" . PHP_EOL);
}
// uploads存在チェック
if (!file_exists($wp_content_dir . "/uploads")) {
  die("wp-contentディレクトリ直下にuploadsディレクトリが存在しません" . PHP_EOL);
}

// DB接続確認
try {
  new PDO('mysql:host=' . $host . ';dbname=' . $db, $user, $pass);
} catch (PDOException $e) {
  die("MySQLに接続出来ません" . PHP_EOL);
}

// 引数でエクスポートディレクトリの指定がある場合はそこに出力する
if ($export_dir) { // 指定があった時はパスチェック
  if (!file_exists($export_dir)) {
    die("指定のエクスポートディレクトリが存在しません" . PHP_EOL);
  }
} else {
  // 引数でエクスポートディレクトリの指定がない場合は本スクリプトと同ディレクトリにする
  $export_dir = __DIR__;
}

/* uploadsディレクトリをzip化する */
// zipコマンドは絶対パスでやるとそこまでのパスが入っちゃうのでそのディレクトリに移動する
// wp-loads.phpを読み込むとWP_CONTENT_DIRが定義されているのでそれを使う
echo_msg("> cd {$wp_content_dir}");
chdir($wp_content_dir);

// -r サブディレクトリも再帰的にzip化する
// -q 動作中のメッセージを表示しない
$zippath = "{$export_dir}/wp_data-" . date("YmdHis") . ".zip";  // データディレクトリに生成するzipパス
exec_cmd("zip -rq {$zippath} ./uploads"); // wp-content直下のuploadsディレクトリをzipに突っ込む

if (!file_exists($zippath)) {
  die("uploadsディレクトリをzip出来ません" . PHP_EOL);
}

/* mysqldumpで指定DBのデータを出力する */
echo_msg("> cd {$export_dir}");
chdir($export_dir);  // uploads手前から移動
$sql_file = "wordpress_db.sql";
// --single-transaction サーバーからデータをダンプする前に BEGIN SQL ステートメントを発行（DBロックしない）
// --no-tablespaces CREATE LOGFILE GROUP ステートメントおよび CREATE TABLESPACE ステートメントを出力に書き出さない（これがないとWARNING出る）
exec_cmd("mysqldump -h {$host} -u {$user} -p{$pass} --single-transaction --no-tablespaces {$db} > {$sql_file}");

if (!file_exists($sql_file)) {
  die("SQLファイルをダンプ出来ません" . PHP_EOL);
}

/* sqlファイルをzipに追加する */
exec_cmd("zip -q {$zippath} {$sql_file}");

/* sqlファイルはzipしたら不要なので消去 */
exec_cmd("rm {$sql_file}");

echo_msg("success : export {$zippath}");

// 最後に改行つけるだけのecho
function echo_msg($msg) {
  echo $msg . PHP_EOL;
}

// コマンド実行関数
function exec_cmd($cmd) {
  echo_msg("> {$cmd}");
  system($cmd, $return);  // exec()ではなくsystem()を使っているのは実行結果を出力してくれるから
  return $return;
}