<?php
/**
 * import_wp_data.php
 * export_wp_data.phpでWordPressの記事や写真のデータをインポートするPHPスクリプトです。
 * このスクリプトは「Search-Replace-DB-master」を使います。これによりDB内のドメインデータが置換されます。
 * https://interconnectit.com/search-and-replace-for-wordpress-databases/
 *
 * ■使い方
 * 本スクリプトをwordpressディレクトリ直下に置くか、ドキュメントルート外に置いて引数でwp-load.phpのパスを指定して実行します。
 *
 * ■引数
 * -z(必須) : export_wp_data.phpでエクスポートして、uploadsディレクトリに配置したzipのファイル名。
 * -w(必須) : wp-contentディレクトリのパス。
 * -h(必須) : MySQLのhostname
 * -u(必須) : MySQLのusername
 * -p(必須) : MySQLのpassword
 * -d(必須) : MySQLのdatabase
 * -s(オプション) : コピー元のWordPressドメイン。
 * -r(オプション) : コピー先のWordPressドメイン。
 * -b(オプション) : バックアップフラグ。フラグがある場合はインポート前にデータをエクスポートする（要export_wp_data.php）
 *
 * ■セキュリティについて
 * 本スクリプトはCLIからしか実行出来ない最低限のセキュリティがありますが完璧ではありません。
 * ドキュメントルート外に置いて引数指定で実行して、役目を終えたらいずれにファイルもすぐに消すとより安全です。
 */

if (php_sapi_name() != 'cli') {
  die("only cli." . PHP_EOL);
}

echo_msg("start import_wp_data");

// まずはこのスクリプトの場所に移動
echo_msg("> cd " . __DIR__);
chdir(__DIR__);

/**
 * パラメータの取得
 */
$options = getopt("z:h:u:p:d:w:s::r::b::");
// 必須パラメータ
$zip_file       = (isset($options["z"])) ? $options["z"] : die("インポートzipファイル名指定は必須です。" . PHP_EOL);
$wp_content_dir = (isset($options["w"])) ? $options["w"] : die("wp-contentディレクトリパスのパラメータは必須です" . PHP_EOL);
$host           = (isset($options["h"])) ? $options["h"] : die("DB接続情報のhostは必須です。" . PHP_EOL);
$user           = (isset($options["u"])) ? $options["u"] : die("DB接続情報のuserは必須です" . PHP_EOL);
$pass           = (isset($options["p"])) ? $options["p"] : die("DB接続情報のpassは必須です" . PHP_EOL);
$db             = (isset($options["d"])) ? $options["d"] : die("DB接続情報のdbは必須です" . PHP_EOL);
// オプションパラメータ
$source_str  = (isset($options["s"])) ? $options["s"] : false;
$replace_str = (isset($options["r"])) ? $options["r"] : false;
$is_backup   = (array_key_exists("b", $options)) ? true : false;  // バックアップフラグはキーがあるかどうか

// wp-contentチェック
$wp_content_dir = realpath($wp_content_dir);
if (!$wp_content_dir) {
  die("wp-contentディレクトリの指定が正しくありません" . PHP_EOL);
}

// zipファイルチェック
$zip_path = $wp_content_dir . "/{$zip_file}";
if (!file_exists($zip_path) || pathinfo($zip_path)["extension"] != "zip") {
  die("wp-contentディレクトリに指定のzipファイルパスが存在しないか、zipファイルではありません" . PHP_EOL);
}

// DB接続確認
try {
  new PDO('mysql:host=' . $host . ';dbname=' . $db, $user, $pass);  // 接続確認出来ない場合は例外となる
} catch (PDOException $e) {
  die("MySQLに接続出来ません" . PHP_EOL);
}

/* バックアップ指定がある時はインポート前に現状のデータをバックアップする */
if ($is_backup) {
  if (!file_exists(__DIR__ . "/export_wp_data.php")) {
    die("バックアップオプションには同ディレクトリにexport_wp_data.phpが必要です" . PHP_EOL);
  }
  // インポート時のバックアップでは現在のディレクトリに置く。バックアップ先を指定するなら-eパラメータで可能
  exec_cmd("php export_wp_data.php -w{$wp_content_dir} -h{$host} -u{$user} -p{$pass} -d{$db}");
}

// zipファイルが存在するディレクトリに移動する
echo_msg("> cd {$wp_content_dir}");
chdir($wp_content_dir);

/* 旧uploadsディレクトリの削除する */
$old_uploads_dir = "{$wp_content_dir}/uploads"; // 一応絶対パス
if (file_exists($old_uploads_dir)) {   // 旧uploadsがある場合は削除する
  exec_cmd("rm -rf {$old_uploads_dir}");
}

/* 新uploadsディレクトリを解凍する */
exec_cmd("unzip -q {$zip_file}");

/* 新uploadsディレクトリはこの実行ユーザーで解凍されるのでアクセス権は変えておく（これは要検討） */
exec_cmd("chmod -R 777 uploads");

/* DBデータのインポート */
exec_cmd("mysql -u {$user} -h {$host} -p{$pass} {$db} < wordpress_db.sql");

/* 解凍したsqlファイルは削除する */
exec_cmd("rm wordpress_db.sql");

/* スクリプトの位置に戻ってSRDBがある場合のみDB内の置換処理をする */
echo_msg("> cd " . __DIR__);
chdir(__DIR__);

$srdb_path = __DIR__ . "/Search-Replace-DB/srdb.cli.php";
if (file_exists($srdb_path) && $source_str && $replace_str) {
  echo_msg("replace database string.");
  exec_cmd("php {$srdb_path} -h='{$host}' -u='{$user}' -p='{$pass}' -n='{$db}' -s='{$source_str}' -r='{$replace_str}'");
} else {
  echo_msg("DB内の文字列置換は実行しません");
}

echo_msg("success : import {$zip_file}");

// 最後に改行つけるだけのecho
function echo_msg($msg) {
  echo $msg . PHP_EOL;
}

// コマンド実行関数
function exec_cmd($cmd) {
  echo_msg("> {$cmd}");
  system($cmd, $return);  // exec()ではなくsystem()を使っているのは実行結果を出力してくれるから
  return $return;
}